#!/usr/bin/python
# -*- coding: utf-8 -*-


import sys
import string
import types
import logging
import logging.handlers
from cmq.cmq_exception import *
import os

METHODS = ["POST", "GET"]
PERMISSION_ACTIONS = ["setqueueattributes", "getqueueattributes", "sendmessage", "receivemessage", "deletemessage", "peekmessage", "changevisibility"]

root_path = sys.path[0]
log_dir_base = root_path + '/logs/'
if not os.path.exists(log_dir_base):
   os.makedirs(log_dir_base)

class CMQLogger:
    @staticmethod
    def get_logger(log_name="CMQ_python_sdk", log_file=log_dir_base+"CMQ_python_sdk.log", log_level=logging.INFO):
        logger = logging.getLogger(log_name)
        if logger.handlers == []:
            fileHandler = logging.handlers.RotatingFileHandler(log_file, maxBytes=10*1024*1024)
            formatter = logging.Formatter('[%(asctime)s] [%(name)s] [%(levelname)s] [%(filename)s:%(lineno)d] [%(thread)d] %(message)s', '%Y-%m-%d %H:%M:%S')
            fileHandler.setFormatter(formatter)
            logger.addHandler(fileHandler)
        CMQLogger.validate_loglevel(log_level)
        logger.setLevel(log_level)
        return logger

    @staticmethod
    def validate_loglevel(log_level):
        log_levels = [logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR, logging.CRITICAL]
        if log_level not in log_levels:
            raise CMQClientParameterException("LogLevelInvalid", "Bad value: '%s', expect levels: '%s'." % \
                (log_level, ','.join([str(item) for item in log_levels])))



#!/usr/bin/python
# -*- coding: utf-8 -*-


import socket
from http.client import HTTPConnection, BadStatusLine, HTTPSConnection
from cmq.cmq_exception import CMQClientNetworkException

class CMQHTTPConnection(HTTPConnection):
    def __init__(self, host, port=None):
        HTTPConnection.__init__(self, host, port)
        self.request_length = 0

    def send(self, astr):
        HTTPConnection.send(self, astr)
        self.request_length += len(astr)

    def request(self, method, url, body=None, headers={}):
        self.request_length = 0
        HTTPConnection.request(self, method, url, body, headers)

class CMQHTTPSConnection(HTTPSConnection):
    def __init__(self, host, port=None):
        HTTPSConnection.__init__(self, host, port)
        self.request_length = 0

    def send(self, astr):
        HTTPSConnection.send(self, astr)
        self.request_length += len(astr)

    def request(self, method, url, body=None, headers={}):
        self.request_length = 0
        HTTPSConnection.request(self, method, url, body, headers)

class CMQHttp:
    def __init__(self, host, connection_timeout = 10, keep_alive = True, logger=None, is_https=False):
        if is_https:
            self.conn = CMQHTTPSConnection(host)
        else:
            self.conn = CMQHTTPConnection(host)
        self.connection_timeout = connection_timeout
        self.keep_alive = keep_alive
        self.request_size = 0
        self.response_size = 0
        self.logger = logger
        if self.logger:
            self.logger.debug("InitCMQHttp KeepAlive:%s ConnectionTime:%s" % (self.keep_alive, self.connection_timeout))

    def set_log_level(self, log_level):
        if self.logger:
            self.logger.setLevel(log_level)

    def close_log(self):
        self.logger = None

    def set_connection_timeout(self, connection_timeout):
        self.connection_timeout = connection_timeout

    def set_keep_alive(self, keep_alive):
        self.keep_alive = keep_alive

    def is_keep_alive(self):
        return self.keep_alive

    def send_request(self, req_inter,UserTimeOut):
        try:
            if self.logger:
                self.logger.debug("SendRequest %s" % req_inter)
            if req_inter.method == 'GET':
                req_inter_url = '%s?%s' % (req_inter.uri, req_inter.data)
                self.conn.request(req_inter.method, req_inter_url, None, req_inter.header)
            elif req_inter.method == 'POST':
                self.conn.request(req_inter.method, req_inter.uri, req_inter.data, req_inter.header)
            else:
                raise Exception('Method only support (GET, POST)')
            self.conn.sock.settimeout(self.connection_timeout+int(UserTimeOut))
            self.conn.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            try:
                http_resp = self.conn.getresponse()
            except BadStatusLine:
                #open another connection when keep-alive timeout
                #httplib will not handle keep-alive timeout, so we must handle it ourself
                if self.logger:
                    self.logger.debug("keep-alive timeout, reopen connection")
                self.conn.close()

                #raise CMQClientNetworkException("Connection TImeout");
                self.conn.request(req_inter.method, req_inter.uri, req_inter.data, req_inter.header)
                self.conn.sock.settimeout(self.connection_timeout+int(UserTimeOut))
                self.conn.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                http_resp = self.conn.getresponse()
            headers = dict(http_resp.getheaders())
            resp_inter = ResponseInternal(status = http_resp.status, header = headers, data = http_resp.read())
            self.request_size = self.conn.request_length
            self.response_size = len(resp_inter.data)
            if not self.is_keep_alive():
                self.conn.close()
            if self.logger:
                self.logger.debug("GetResponse %s" % resp_inter)
            return resp_inter
        except Exception as e:
            self.conn.close()
            raise CMQClientNetworkException(str(e))

class RequestInternal:
    def __init__(self, method = "", uri = "", header = None, data = ""):
        if header == None:
            header = {}
        self.method = method
        self.uri = uri
        self.header = header
        self.data = data

    def __str__(self):
        return "Method: %s\nUri: %s\nHeader: %s\nData: %s\n" % \
                (self.method, self.uri, "\n".join(["%s: %s" % (k,v) for k,v in list(self.header.items())]), self.data)

class ResponseInternal:
    def __init__(self, status = 0, header = None, data = ""):
        if header == None:
            header = {}
        self.status = status
        self.header = header
        self.data = data

    def __str__(self):
        return "Status: %s\nHeader: %s\nData: %s\n" % \
            (self.status, "\n".join(["%s: %s" % (k,v) for k,v in list(self.header.items())]), self.data)



#!/usr/bin/python
# -*- coding: utf-8 -*-


import time
import base64

class Queue:
    def __init__(self, queue_name, cmq_client, debug=False): 
        self.queue_name = queue_name
        self.cmq_client = cmq_client
        self.set_encoding(False)
        self.debug = debug

    def set_debug(self, debug):
        self.debug = debug

    def set_encoding(self, encoding):
        """ 设置是否对消息体进行base64编码
 
            @type encoding: bool
            @param encoding: 是否对消息体进行base64编码
        """
        self.encoding = encoding
        
    def create(self, queue_meta):
        """ 创建队列

            @type queue_meta: QueueMeta object
            @param queue_meta: QueueMeta对象，设置队列的属性
        """
        params = {
            'queueName':self.queue_name,
            'pollingWaitSeconds':queue_meta.pollingWaitSeconds,
            'visibilityTimeout':queue_meta.visibilityTimeout,
            'maxMsgSize':queue_meta.maxMsgSize,
            'msgRetentionSeconds':queue_meta.msgRetentionSeconds,
            'rewindSeconds':queue_meta.rewindSeconds
        }
        if queue_meta.maxMsgHeapNum > 0:
            params['maxMsgHeapNum'] = queue_meta.maxMsgHeapNum
        self.cmq_client.create_queue(params)

    def rewindQueue(self, backTrackingTime):
        '''
        rewind queue
        @type backTrackingTime the back tracking time 
        '''
        params = {
              'queueName' : self.queue_name,
              'startConsumeTime':backTrackingTime
        }
        self.cmq_client.rewindQueue(params)
    
    def get_attributes(self):
        """ 获取队列属性

            @rtype: QueueMeta object
            @return 队列的属性
        """
        params = {
            'queueName':self.queue_name
        }
        resp = self.cmq_client.get_queue_attributes(params)
        queue_meta = QueueMeta()
        queue_meta.queueName = self.queue_name
        self.__resp2meta__(queue_meta, resp)
        return queue_meta

    def set_attributes(self, queue_meta):
        """ 设置队列属性

            @type queue_meta: QueueMeta object
            @param queue_meta: QueueMeta对象，设置队列的属性
        """
        params = {
            'queueName':self.queue_name,
            'pollingWaitSeconds':queue_meta.pollingWaitSeconds,
            'visibilityTimeout':queue_meta.visibilityTimeout,
            'maxMsgSize':queue_meta.maxMsgSize,
            'msgRetentionSeconds':queue_meta.msgRetentionSeconds,
            'rewindSeconds':queue_meta.rewindSeconds
        }
        if queue_meta.maxMsgHeapNum > 0:
            params['maxMsgHeapNum'] = queue_meta.maxMsgHeapNum
            
        self.cmq_client.set_queue_attributes(params)
            
            
    def delete(self):
        """ 删除队列

        """
        params = {'queueName':self.queue_name}
        self.cmq_client.delete_queue(params)

    def send_message(self, message,delayTime = 0):
        """ 发送消息

            @type message: Message object
            @param message: 发送的Message object

            @rtype: Message object
            @return 消息发送成功的返回属性，包含MessageId

        """
        if self.encoding:
            msgBody = base64.b64encode(message.msgBody)
        else:
            msgBody = message.msgBody
        params = {
            'queueName':self.queue_name,
            'msgBody':msgBody,
            'delaySeconds':delayTime,
        }
        msgId = self.cmq_client.send_message(params)
        retmsg = Message()
        retmsg.msgId = msgId
        return retmsg

    def batch_send_message(self, messages, delayTime = 0):
        """ 批量发送消息
           
           @type messages: list of Message object
           @param messages: 发送的Message object list

           @rtype: list of Message object
           @return 多条消息发送成功的返回属性，包含MessageId
        """
        params = {
            'queueName':self.queue_name,
            'delaySeconds':delayTime,
        }
        n = 1
        for message in messages:
            key = 'msgBody.%d' % n
            if self.encoding:
                params[key] = base64.b64encode(message.msgBody)
            else:
                params[key] = message.msgBody
            n += 1
        msgList = self.cmq_client.batch_send_message(params)
        retMessageList = []
        for msg in msgList:
            retmsg = Message()
            retmsg.msgId = msg['msgId']
            retMessageList.append(retmsg)
        return retMessageList
    

    def receive_message(self, polling_wait_seconds=None):
        """ 消费消息

            @type polling_wait_seconds: int
            @param polling_wait_seconds: 本次请求的长轮询时间，单位：秒

            @rtype: Message object
            @return Message object中包含基本属性、临时句柄
        """
        params = {'queueName':self.queue_name}
        if polling_wait_seconds != None:
            params['UserpollingWaitSeconds'] = polling_wait_seconds
            params['pollingWaitSeconds'] = polling_wait_seconds
        else:
            params['UserpollingWaitSeconds'] = str(30)
        resp = self.cmq_client.receive_message(params)
        msg = Message()
        if self.encoding:
            msg.msgBody = base64.b64decode(resp['msgBody'])
        else:
            msg.msgBody = resp['msgBody']
        msg.msgId = resp['msgId']
        msg.receiptHandle = resp['receiptHandle']
        msg.enqueueTime = resp['enqueueTime']
        msg.nextVisibleTime = resp['nextVisibleTime']
        msg.dequeueCount = resp['dequeueCount']
        msg.firstDequeueTime = resp['firstDequeueTime']
        return msg
   
    def batch_receive_message(self, num_of_msg, polling_wait_seconds=None):
        """ 批量消费消息

            @type num_of_msg: int
            @param num_of_msg: 本次请求最多获取的消息条数

            @type polling_wait_seconds: int
            @param polling_wait_seconds: 本次请求的长轮询时间，单位：秒

            @rtype: list of Message object
            @return 多条消息的属性，包含消息的基本属性、临时句柄
        """
        params = {'queueName':self.queue_name, 'numOfMsg':num_of_msg}
        if polling_wait_seconds != None:
            params['UserpollingWaitSeconds'] = polling_wait_seconds
            params['pollingWaitSeconds'] = polling_wait_seconds
        else:
            params['UserpollingWaitSeconds'] = str(30)
        msgInfoList = self.cmq_client.batch_receive_message(params)
        retMessageList = []
        for msg in msgInfoList:
            retmsg = Message()
            if self.encoding:
                retmsg.msgBody = base64.b64decode(msg['msgBody'])
            else:
                retmsg.msgBody = msg['msgBody']
            retmsg.msgId = msg['msgId']
            retmsg.receiptHandle = msg['receiptHandle']
            retmsg.enqueueTime = msg['enqueueTime']
            retmsg.nextVisibleTime = msg['nextVisibleTime']
            retmsg.dequeueCount = msg['dequeueCount']
            retmsg.firstDequeueTime = msg['firstDequeueTime']
            retMessageList.append(retmsg)
        return retMessageList

    def delete_message(self, receipt_handle):
        """ 删除消息

            @type receipt_handle: string
            @param receipt_handle: 最近一次操作该消息返回的临时句柄
        """
        params = {'queueName':self.queue_name, 'receiptHandle':receipt_handle}
        self.cmq_client.delete_message(params)

    def batch_delete_message(self, receipt_handle_list):
        """批量删除消息
            
            @type receipt_handle_list: list
            @param receipt_handle_list: batch_receive_message返回的多条消息的临时句柄
        """
        params = {'queueName':self.queue_name}
        n = 1
        for receipt_handle in receipt_handle_list:
            key = 'receiptHandle.%d' % n
            params[key] = receipt_handle
            n += 1
        self.cmq_client.batch_delete_message(params)

    
    def __resp2meta__(self, queue_meta, resp):
        if 'queueName' in list(resp.keys()):
            queue_meta.queueName = resp['queueName']
        if 'maxMsgHeapNum' in list(resp.keys()):
            queue_meta.maxMsgHeapNum = resp['maxMsgHeapNum']
        if 'pollingWaitSeconds' in list(resp.keys()):
            queue_meta.pollingWaitSeconds = resp['pollingWaitSeconds']
        if 'visibilityTimeout' in list(resp.keys()):
            queue_meta.visibilityTimeout = resp['visibilityTimeout']
        if 'maxMsgSize' in list(resp.keys()):
            queue_meta.maxMsgSize = resp['maxMsgSize']
        if 'msgRetentionSeconds' in list(resp.keys()):
            queue_meta.msgRetentionSeconds = resp['msgRetentionSeconds']
        if 'createTime' in list(resp.keys()):
            queue_meta.createTime = resp['createTime']
        if 'lastModifyTime' in list(resp.keys()):
            queue_meta.lastModifyTime = resp['lastModifyTime']
        if 'activeMsgNum' in list(resp.keys()):
            queue_meta.activeMsgNum = resp['activeMsgNum']
        if 'inactiveMsgNum' in list(resp.keys()):
            queue_meta.inactiveMsgNum = resp['inactiveMsgNum']
        if 'rewindSeconds' in list(resp.keys()):
            queue_meta.rewindSeconds = resp['rewindSeconds']
        if 'rewindmsgNum' in list(resp.keys()):
            queue_meta.rewindmsgNum = resp['rewindmsgNum']
        if 'minMsgTime' in list(resp.keys()):
            queue_meta.minMsgTime = resp['minMsgTime']
        if 'delayMsgNum' in list(resp.keys()):
            queue_meta.delayMsgNum = resp['delayMsgNum']

class QueueMeta:
    # DEFAULT_MAX_MSG_HEAP_NUM = 30
    DEFAULT_POLLING_WAIT_SECONDS = 0
    DEFAULT_VISIBILITY_TIMEOUT = 30
    DEFAULT_MAX_MSG_SIZE = 65536
    DEFAULT_MSG_RETENTION_SECONDS = 345600
    def __init__(self):
        """ 队列属性
            @note: 设置属性
            :: maxMsgHeapNum: 最大堆积消息数
            :: pollingWaitSeconds: receive message时，长轮询时间，单位：秒
            :: visibilityTimeout: 消息可见性超时, 单位：秒
            :: maxMsgSize: 消息最大长度, 单位：Byte
            :: msgRetentionSeconds: 消息保留周期，单位：秒
            :: rewindSeconds:消息回溯时间，单位：秒， 取值范围0-43200
            
            @note: 非设置属性
            :: activeMsgNum: 可消费消息数，近似值
            :: inactiveMsgNum: 正在被消费的消息数，近似值
            :: createTime: queue创建时间，单位：秒 
            :: lastModifyTime: 修改queue属性的最近时间，单位：秒
            :: queue_name: 队列名称
            :: rewindmsgNum: 已删除，但是还在回溯保留时间内的消息数量
            :: minMsgTime: 消息最小未消费时间，单位：秒
            :: delayMsgNum: 延时消息数量
        """
        
        self.queueName = ""
        self.maxMsgHeapNum = -1
        self.pollingWaitSeconds = QueueMeta.DEFAULT_POLLING_WAIT_SECONDS
        self.visibilityTimeout = QueueMeta.DEFAULT_VISIBILITY_TIMEOUT
        self.maxMsgSize = QueueMeta.DEFAULT_MAX_MSG_SIZE
        self.msgRetentionSeconds = QueueMeta.DEFAULT_MSG_RETENTION_SECONDS
        self.createTime = -1
        self.lastModifyTime = -1
        self.activeMsgNum = -1
        self.inactiveMsgNum = -1
        self.rewindSeconds  = 0
        self.rewindmsgNum = 0
        self.minMsgTime = 0
        self.delayMsgNum = 0 

    def __str__(self):
        meta_info = {"visibilityTimeout" : self.visibilityTimeout,
                     "maxMsgHeapNum" : self.maxMsgHeapNum,
                     "maxMsgSize" : self.maxMsgSize,
                     "msgRetentionSeconds" : self.msgRetentionSeconds,
                     "pollingWaitSeconds" : self.pollingWaitSeconds,
                     "activeMsgNum" : self.activeMsgNum,
                     "inactiveMsgNum" : self.inactiveMsgNum,
                     "createTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.createTime)),
                     "lastModifyTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.lastModifyTime)),
                     "QueueName" : self.queueName,
                     "rewindSeconds":self.rewindSeconds,
                     "rewindmsgNum":self.rewindmsgNum,
                     "minMsgTime":self.minMsgTime,
                     "delayMsgNum":self.delayMsgNum}
        return "\n".join(["%s: %s" % (k.ljust(30), v) for k, v in list(meta_info.items())])

class Message:
    def __init__(self, message_body="", message_tag=[]):
        """ 消息属性

            @note: send_message 指定属性
            :: msgBody         消息体 

            @note: send_message 返回属性
            :: msgId           消息编号

            @note: receive_message 返回属性，除基本属性外
            :: receiptHandle       下次删除或修改消息的临时句柄
            :: enqueueTime         消息入队时间
            :: nextVisibleTime     下次可被再次消费的时间
            :: dequeueCount        总共被消费的次数
            :: firstDequeueTime    第一次被消费的时间
            :: msgTag              消息标签
        """
        self.msgBody = message_body
        self.msgId = ""
        self.enqueueTime = -1
        self.receiptHandle = ""
        self.nextVisibleTime = -1
        self.dequeueCount = -1
        self.firstDequeueTime = -1
        self.msgTag = message_tag

    def __str__(self):
        msg_info = {"msgBody" : self.msgBody,
                     "msgId" : self.msgId,
                     "enqueueTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.enqueueTime)),
                     "nextVisibleTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.nextVisibleTime)),
                     "firstDequeueTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.firstDequeueTime)),
                     "dequeueCount" : self.dequeueCount,
                     "receiptHandle" : self.receiptHandle,
                     "msgTag":self.msgTag}
        return "\n".join(["%s: %s" % (k.ljust(30), v) for k, v in list(msg_info.items())])
    
    



#!/usr/bin/env python
# coding=utf-8

import time
import base64

class Topic:

    def __init__(self, topicName, cmq_client,
                 debug=False):
        self.topicName = topicName
        self.cmq_client = cmq_client
        self.set_encoding(False)
        self.debug = debug
    
    def set_debug(self, debug):
        self.debug = debug


    def set_encoding(self, encoding):
        self.encoding = encoding
        
        
    def create(self, topic_meta):
        
        """ create topic

            @type topic_meta: TopicMeta object
        """
        params = {
            'topicName':self.topicName,
            'filterType':topic_meta.filterType,
        }
        
        if topic_meta.maxMsgSize > 0:
            params['maxMsgSize'] = topic_meta.maxMsgSize
        self.cmq_client.create_topic(params)
        
    
            
    def delete(self):
        ''' delete opic
               
        '''
        params = {
            "topicName" : self.topicName,
            }
        self.cmq_client.delete_topic(params)
        
    def get_attributes(self):
        """ get topic attributes 
            @return:  TopicMeta ojbect
        """
        params = {
            'topicName':self.topicName
        }
        resp = self.cmq_client.get_topic_attributes(params)
        topic_meta = TopicMeta()
        topic_meta.topicName = self.topicName
        self.__resp2meta__(topic_meta, resp)
        return topic_meta
    
    def set_attributes(self, topic_meta):
        """ set topic attributes
             @type TopicMeta object
        """
        params = {
            'topicName':self.topicName,
        }
        if topic_meta.maxMsgSize > 0:
            params['maxMsgSize'] = topic_meta.maxMsgSize
            
        self.cmq_client.set_topic_attributes(params)
        
    def publish_message(self, message, routingKey=""):
        """ send message

            @type message: Message object
            @param message: 发送的Message object

            @rtype: Message object
            @return 消息发送成功的返回属性，包含MessageId

        """
        if self.encoding:
            msgBody = base64.b64encode(message.msgBody)
        else:
            msgBody = message.msgBody
        params = {
            'topicName':self.topicName,
            'msgBody':msgBody,
            'routingKey':routingKey
        }
        # for tags
         
        if message.msgTag:
            n = 1
            for tag in message.msgTag:
                key = 'msgTag.%d' % n
                params[key] = tag
                n += 1
            
        msgId = self.cmq_client.publish_message(params)
        retmsg = Message()
        retmsg.msgId = msgId
        return retmsg

    def batch_publish_message(self, messages,msgtag=[],routingKey= ""):
        """ 批量发送消息
           
           @type messages: list of Message object
           @param messages: 发送的Message object list

           @rtype: list of Message object
           @return 多条消息发送成功的返回属性，包含MessageId
        """
        params = {
            'topicName':self.topicName,
            'routingKey':routingKey,
        }
        
        n = 1
        for message in messages:
            msg = 'msgBody.%d' % n
            if self.encoding:
                params[msg] = base64.b64encode(message.msgBody)
            else:
                params[msg] = message.msgBody
            n+=1

        n = 1 
        for tag in msgtag:
            key = 'msgTag.%d' % n
            params[key] = tag
            n += 1 

        msgList = self.cmq_client.batch_publish_message(params)
        retMessageList = []
        for msg in msgList:
            retmsg = Message()
            retmsg.msgId = msg['msgId']
            retMessageList.append(retmsg)
        return retMessageList

    def list_subscription(self, searchWord="" , limit=-1, offset=""):
        '''列出Topic的订阅

            @type searchWord: string
            @param searchWord: 关键字

            @type limit: int
            @param limit: list_subscription最多返回的订阅数

            @type offset: string
            @param offset: list_subscription的起始位置，上次list_subscription返回的next_offset

            @rtype: tuple
            @return: subscriptionURL的列表和下次list subscription的起始位置; 如果所有subscription都list出来，next_offset为"".
        '''   
        params = {
            'topicName':self.topicName,
        }
        
        if searchWord != "":
            params['searchWord'] = searchWord;
        
        if limit != -1:
            params['limit'] = limit;
        
        if offset != "":
            params['offset'] = offset
        
        ret_pkg = self.cmq_client.list_subscription(params)
        
        if offset == "":
            next_offset = len(ret_pkg['subscriptionList'])
        else:
            next_offset = offset + len(ret_pkg['subscriptionList'])
        
        if next_offset >= ret_pkg['totalCount']:
            next_offset = ""
        
        return (ret_pkg['totalCount'], ret_pkg['subscriptionList'], next_offset)
      
    def __resp2meta__(self, topic_meta, resp):
        if 'msgCount' in list(resp.keys()):
            topic_meta.msgCount = resp['msgCount']
        if 'maxMsgSize' in list(resp.keys()):
            topic_meta.maxMsgSize = resp['maxMsgSize']
        if 'msgRetentionSeconds' in list(resp.keys()):
            topic_meta.msgRetentionSeconds = resp['msgRetentionSeconds']
        if 'createTime' in list(resp.keys()):
            topic_meta.createTime = resp['createTime']
        if 'lastModifyTime' in list(resp.keys()):
            topic_meta.lastModifyTime = resp['lastModifyTime']
        if 'filterType' in list(resp.keys()):
            topic_meta.filterType = resp['filterType']

        
class TopicMeta:
    '''
       :: topicName               topic name 
       :: msgCount                消息统计
       :: maxMsgSize              消息最大长度  可修改
       :: msgRetentionSeconds     消息保留时间
       :: createTime              创建时间
       :: lastModifyTime          上次修改实现
    '''
    def __init__(self):
        self.topicName = ""
        self.msgCount = 0
        self.maxMsgSize = 65536
        self.msgRetentionSeconds = 0
        self.createTime = 0
        self.lastModifyTime = 0
        self.filterType = 1
		
    def __str__(self):
        msg_info = { "topicName" : self.topicName,
                     "msgCount" : self.msgCount,
                     "createTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.createTime)),
                     "lastModifyTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.lastModifyTime)),
                     "maxMsgSize" : self.maxMsgSize,
                     "msgRetentionSeconds" : self.msgRetentionSeconds,
                     "filterType": self.filterType}
        return "\n".join(["%s: %s" % (k.ljust(30), v) for k, v in list(msg_info.items())])



class Message:
    def __init__(self, message_body="", message_tag=[]):
        """ 消息属性

            @note: send_message 指定属性
            :: msgBody         消息体 

            @note: send_message 返回属性
            :: msgId           消息编号

            @note: receive_message 返回属性，除基本属性外
            :: receiptHandle       下次删除或修改消息的临时句柄
            :: msgTag              消息标签
        """
        self.msgBody = message_body
        self.msgId = ""
        self.msgTag = message_tag

    def __str__(self):
        msg_info = {"msgBody" : self.msgBody,
                     "msgId" : self.msgId,
                     "msgTag":self.msgTag}
        return "\n".join(["%s: %s" % (k.ljust(30), v) for k, v in list(msg_info.items())])
    





#!/usr/bin/python
# -*- coding: utf-8 -*-

import urllib.request, urllib.parse, urllib.error
import copy
import random
from cmq.cmq_exception import *
from cmq.cmq_tool import CMQLogger
from cmq.cmq_http import CMQHttp, RequestInternal
from .sign import Sign
import json
import time
import sys

URISEC = '/v2/index.php'

class CMQClient:
    def __init__(self, host, secretId, secretKey, version="SDK_Python_1.3", logger=None):
        self.host, self.is_https = self.process_host(host)
        self.secretId = secretId
        self.secretKey = secretKey
        self.version = version
        self.logger = CMQLogger.get_logger() if logger is None else logger
        self.http = CMQHttp(self.host, logger=logger, is_https=self.is_https)
        self.sign_method = 'sha1'
        if self.logger:
            self.logger.debug("InitClient Host:%s Version:%s" % (host, version))
        self.method = 'POST'

    def set_method(self, method='POST'):
        """
        method: POST OR GET
        """
        self.method = method.upper()
    def set_sign_method(self, sign_method='sha1'):
        '''
        @function : set sign method , and current support sha1 and sha256 method 
        @sign_method :  sign method support sha1 and sha256
        @return none or exception for sign method 
        '''
        if sign_method == 'sha1' or sign_method == 'sha256':
            self.sign_method = sign_method
        else:
            raise CMQClientParameterException('Only support sha1 or sha256. invalid method:%s' % sign_method)
        
    def set_log_level(self, log_level):
        if self.logger:
            CMQLogger.validate_loglevel(log_level)
            self.logger.setLevel(log_level)
            self.http.set_log_level(log_level)

    def close_log(self):
        self.logger = None
        self.http.close_log()

    def set_connection_timeout(self, connection_timeout):
        self.http.set_connection_timeout(connection_timeout)

    def set_keep_alive(self, keep_alive):
        self.http.set_keep_alive(keep_alive)

    def close_connection(self):
        self.http.conn.close()

    def process_host(self, host):
        if host.startswith("http://"):
            if host.endswith("/"):
                host = host[:-1]
            host = host[len("http://"):]
            return host, False
        elif host.startswith("https://"):
            if host.endswith("/"):
                host = host[:-1]
            host = host[len("https://"):]
            return host, True
        else:
            raise CMQClientParameterException("Only support http(s) prototol. Invalid host:%s" % host)

    def build_req_inter(self, action, params, req_inter):
        _params = copy.deepcopy(params)
        _params['Action'] = action[0].upper() + action[1:]
        _params['RequestClient'] = self.version

        if (('SecretId' in _params) != True):
            _params['SecretId'] = self.secretId

        if (('Nonce' in _params) != True):
            _params['Nonce'] = random.randint(1, sys.maxsize)

        if (('Timestamp' in _params) != True):
            _params['Timestamp'] = int(time.time())
            
        if (('SignatureMethod' in _params) != True):
            if self.sign_method == 'sha256':
                _params['SignatureMethod'] = 'HmacSHA256'
            else:
                _params['SignatureMethod'] = 'HmacSHA1'
                
        sign = Sign(self.secretId, self.secretKey)
        _params['Signature'] = sign.make(self.host, req_inter.uri, _params, req_inter.method, self.sign_method)

        req_inter.data = urllib.parse.urlencode(_params)

        self.build_header(req_inter)

    def build_header(self, req_inter):
        if self.http.is_keep_alive():
            req_inter.header["Connection"] = "Keep-Alive"

    def check_status(self, resp_inter):
        if resp_inter.status != 200:
            raise CMQServerNetworkException(resp_inter.status, resp_inter.header, resp_inter.data)

        resp = json.loads(resp_inter.data)
        code, message, requestId = resp['code'], resp['message'], resp.get('requestId', '')

        if code != 0:
            raise CMQServerException(message=message, request_id=requestId, code=code, data=resp)

    def request(self, action, params):
        # make request internal
        UserTimeout = '0'
        req_inter = RequestInternal(self.method, URISEC)
        self.build_req_inter(action, params, req_inter)

        # send request
        if None != params.get('UserpollingWaitSeconds'):
            UserTimeout = params.get('UserpollingWaitSeconds')
        resp_inter = self.http.send_request(req_inter, UserTimeout)

        # handle result, make response
        # self.check_status(resp_inter)

        return resp_inter

#===============================================queue operation===============================================#

    def create_queue(self, params):
        resp_inter = self.request('CreateQueue', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("CreateQueue RequestId:%s QueueName:%s QueueId:%s" % \
                (ret['requestId'], params['queueName'], ret['queueId']))

    def delete_queue(self, params):
        resp_inter = self.request('DeleteQueue', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("DeleteQueue RequestId:%s QueueName:%s" % \
                (ret['requestId'], params['queueName']))

    def list_queue(self, params):
        resp_inter = self.request('ListQueue', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("ListQueue RequestId:%s QueueTotalCount:%d" % (ret['requestId'], ret['totalCount']))
        return ret

    def rewindQueue(self, params):
        resp_inter = self.request('RewindQueue', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("RewindQueue RequestId:%s QueueName:%s" % \
                (ret['requestId'], params['queueName']))


    def set_queue_attributes(self, params):
        resp_inter = self.request('SetQueueAttributes', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("SetQueueAttributes RequestId:%s QueueName:%s" % \
                (ret['requestId'], params['queueName']))


    def get_queue_attributes(self, params):
        resp_inter = self.request('GetQueueAttributes', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("GetQueueAttributes RequestId:%s QueueName:%s" % \
                (ret['requestId'], params['queueName']))
        return ret

    def send_message(self, params):
        resp_inter = self.request('SendMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("SendMessage RequestId:%s QueueName:%s MsgId:%s" % \
                (ret['requestId'], params['queueName'], ret['msgId']))
        return ret['msgId']

    def batch_send_message(self, params):
        resp_inter = self.request('BatchSendMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("BatchSendMessage RequestId:%s QueueName:%s MessageCount:%s MessageInfo\n%s" % \
                (ret['requestId'], params['queueName'], len(ret['msgList']), \
                 "\n".join(["MessageId:%s" % (msg['msgId']) for msg in ret['msgList']])))
        return ret['msgList']

    def receive_message(self, params):
        resp_inter = self.request('ReceiveMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("ReceiveMessage RequestId:%s QueueName:%s PollingWaitSeconds:%s MsgId:%s ReceiptHandle:%s \
            EnqueueTime:%s NextVisibleTime:%s FirstDequeueTime:%s DequeueCount:%s" % \
                (ret['requestId'], params['queueName'], params.get('pollingWaitSeconds', ''), ret['msgId'], ret['receiptHandle'],
                 ret['enqueueTime'], ret['nextVisibleTime'], ret['firstDequeueTime'], ret['dequeueCount']))
        return ret

    def batch_receive_message(self, params):
        resp_inter = self.request('BatchReceiveMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("BatchReceiveMessage RequestId:%s QueueName:%s PollingWaitSeconds:%s BatchSize:%s MessageCount:%s MessageInfo\n%s" % \
                (ret['requestId'], params['queueName'], params.get('pollingWaitSeconds', ''), params['numOfMsg'], len(ret['msgInfoList']), \
                 "\n".join(["MessageId:%s ReceiptHandle:%s EnqueueTime:%s NextVisibleTime:%s FirstDequeueTime:%s DequeueCount:%s" % \
                            (msg['msgId'], msg['receiptHandle'], msg['enqueueTime'], msg['nextVisibleTime'], msg['firstDequeueTime'], msg['dequeueCount']) for msg in ret['msgInfoList']])))
        return ret['msgInfoList']

    def delete_message(self, params):
        resp_inter = self.request('DeleteMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("DeleteMessage RequestId:%s QueueName:%s ReceiptHandle:%s" % \
                (ret['requestId'], params['queueName'], params['receiptHandle']))


    def batch_delete_message(self, params):
        resp_inter = self.request('BatchDeleteMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("BatchDeleteMessage RequestId:%s QueueName:%s ReceiptHandles\n%s" % \
                (ret['requestId'], params['queueName'], "\n".join([str(params[key]) for key in list(params.keys()) if 'receiptHandle.' in key])))

#=======================================================topic operation====================================#
    def create_topic(self, params):
        resp_inter = self.request('CreateTopic', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("CreateTopic RequestId:%s TopicName:%s TopicId:%s" % \
                (ret['requestId'], params['topicName'], ret['topicId']))

    def delete_topic(self, params):
        resp_inter = self.request('DeleteTopic', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("DeleteTopic RequestId:%s TopicName:%s" % \
                (ret['requestId'], params['topicName']))

    def list_topic(self, params):
        resp_inter = self.request('ListTopic', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("ListTopic RequestId:%s TopicTotalCount:%d" % (ret['requestId'], ret['totalCount']))
        return ret


    def set_topic_attributes(self, params):
        resp_inter = self.request('SetTopicAttributes', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("SetTopicAttributes RequestId:%s TopicName:%s" % \
                (ret['requestId'], params['topicName']))


    def get_topic_attributes(self, params):
        resp_inter = self.request('GetTopicAttributes', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("GetTopicAttributes RequestId:%s TopicName:%s" % \
                (ret['requestId'], params['topicName']))
        return ret  

    def publish_message(self, params):
        resp_inter = self.request('PublishMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("PublishMessage RequestId:%s TopicName:%s MsgId:%s" % \
                (ret['requestId'], params['topicName'], ret['msgId']))
        return ret['msgId']

    def batch_publish_message(self, params):
        resp_inter = self.request('BatchPublishMessage', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("BatchPublishMessage RequestId:%s TopicName:%s MessageCount:%s MessageInfo\n%s" % \
                (ret['requestId'], params['topicName'], len(ret['msgList']), \
                 "\n".join(["MessageId:%s" % (msg['msgId']) for msg in ret['msgList']])))
        return ret['msgList']  
#========================================Subscription operation===========================================#
    def create_subscription(self, params):
        resp_inter = self.request('Subscribe', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("Create subscription RequestId:%s subscription Name:%s subscripton Id:%s" % \
                (ret['requestId'], params['subscriptionName'], ret['subscriptionId']))

    def delete_subscription(self, params):
        resp_inter = self.request('Unsubscribe', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("DeleteTopic RequestId:%s TopicName:%s" % \
                (ret['requestId'], params['topicName']))

    def clear_filterTags(self, params):
        resp_inter = self.request('ClearSubscriptionFilterTags', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("ClearSubscriptionFilterTags RequestId:%s TopicName:%s" % \
                (ret['requestId'], params['topicName']))


    def list_subscription(self, params):
        resp_inter = self.request('ListSubscriptionByTopic', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("ListTopic RequestId:%s TopicTotalCount:%d" % (ret['requestId'], ret['totalCount']))
        return ret


    def set_subscription_attributes(self, params):
        resp_inter = self.request('SetSubscriptionAttributes', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("Set subscription Attributes RequestId:%s TopicName:%s" % \
                (ret['requestId'] if 'requestId' in ret else '', params['topicName'] if 'topicName' in ret else '' ))


    def get_subscription_attributes(self, params):
        resp_inter = self.request('GetSubscriptionAttributes', params)
        self.check_status(resp_inter)

        ret = json.loads(resp_inter.data)
        if self.logger:
            self.logger.debug("GetTopicAttributes RequestId:%s TopicName:%s" % \
                (ret['requestId'], params['topicName']))
        return ret  






#!/usr/bin/python
# -*- coding: utf-8 -*-

import binascii
import hashlib
import hmac

class Sign:
    def __init__(self, secretId, secretKey):
        self.secretId = secretId
        self.secretKey = secretKey

    def make(self, requestHost, requestUri, params, method='POST', sign_method='sha1'):
        new_params = {}
        for param_key in params:
            if method.upper() == 'POST' and str(params[param_key])[0:1] == "@":
                continue
            new_params[param_key] = params[param_key]
        srcStr = method.upper() + requestHost + requestUri + '?' + "&".join(k.replace("_", ".") + "=" + str(new_params[k]) for k in sorted(new_params.keys()))
        if sign_method == 'sha1':
            hashed = hmac.new(str.encode(self.secretKey), srcStr.encode('utf-8'), hashlib.sha1)
        elif sign_method == 'sha256':
            hashed = hmac.new(str.encode(self.secretKey), srcStr.encode('utf-8'), hashlib.sha256)
        return binascii.b2a_base64(hashed.digest())[:-1]
      
def main():
    secretId = 123
    secretKey = 'xxx'
    params = {'a':1, 'b':2}
    sign = Sign(secretId, secretKey)
    print(sign.make('cmq-gz.api.qcloud.com', '/v2/index.php', params))

if (__name__ == '__main__'):
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from cmq.cmq_client import CMQClient
from .queue import Queue
from cmq.cmq_tool import CMQLogger
from cmq.topic import Topic
from cmq.subscription import Subscription
class Account:
    """
    Account类对象不是线程安全的，如果多线程使用，需要每个线程单独初始化Account类对象
    """
    def __init__(self, host, secretId, secretKey, debug=False):
        """
            @type host: string
            @param host: 访问的url，例如：https://cmq-queue-gz.api.qcloud.com

            @type secretId: string
            @param secretId: 用户的secretId, 腾讯云官网获取

            @type secretKey: string
            @param secretKey: 用户的secretKey，腾讯云官网获取

            @note: Exception
            :: CMQClientParameterException host格式错误
        """
        self.secretId = secretId
        self.secretKey = secretKey
        self.debug = debug
        self.logger = CMQLogger.get_logger()
        self.cmq_client = CMQClient(host, secretId, secretKey, logger=self.logger)
        
    def set_sign(self, sign='sha256'):
        '''
          @fucntion set_sign : set sign method 
          @sign  sha256 or sha1 
        '''
        self.cmq_client.set_sign_method(sign)
    def set_debug(self, debug):
        self.debug = debug

    def set_log_level(self, log_level):
        """ 设置logger的日志级别
            @type log_level: int
            @param log_level: one of logging.DEBUG,logging.INFO,logging.WARNING,logging.ERROR,logging.CRITICAL
        """
        CMQLogger.validate_loglevel(log_level)
        self.logger.setLevel(log_level)
        self.cmq_client.set_log_level(log_level)

    def close_log(self):
        """ 关闭日志打印
        """
        self.cmq_client.close_log()


    def set_client(self, host, secretId=None, secretKey=None):
        """ 设置访问的url

            @type host: string
            @param host: 访问的url，例如：http://cmq-queue-gz.api.tencentyun.com

            @type secretId: string
            @param secretId: 用户的secretId，腾讯云官网获取

            @type secretKey: string
            @param secretKey: 用户的secretKey，腾讯云官网获取

            @note: Exception
            :: CMQClientParameterException host格式错误
        """
        if secretId is None:
            secretId = self.secretId
        if secretKey is None:
            secretKey = self.secretKey
        self.cmq_client = CMQClient(host, secretId, secretKey, logger=self.logger)

    def get_queue(self, queue_name):
        """ 获取Account的一个Queue对象

            @type queue_name: string
            @param queue_name: 队列名

            @rtype: Queue object
            @return: 返回该Account的一个Queue对象
        """
        return Queue(queue_name, self.cmq_client, self.debug)

    def get_client(self):
        """ 获取queue client

            @rtype: CMQClient object
            @return: 返回使用的CMQClient object
        """
        return self.cmq_client

    def list_queue(self, searchWord="", limit=-1, offset=""):
        
        """ 列出Account的队列

            @type searchWord: string
            @param searchWord: 队列名的前缀

            @type limit: int
            @param limit: list_queue最多返回的队列数

            @type offset: string
            @param offset: list_queue的起始位置，上次list_queue返回的next_offset

            @rtype: tuple
            @return: QueueURL的列表和下次list queue的起始位置; 如果所有queue都list出来，next_offset为"".
        """
        params = {}
        if searchWord != "":
            params['searchWord'] = searchWord
        if limit != -1:
            params['limit'] = limit
        if offset != "":
            params['offset'] = offset
        ret_pkg = self.cmq_client.list_queue(params)

        if offset == "":
            next_offset = len(ret_pkg['queueList'])
        else:
            next_offset = offset + len(ret_pkg['queueList'])
        if next_offset >= ret_pkg['totalCount']:
            next_offset = ""

        return (ret_pkg['totalCount'], ret_pkg['queueList'], next_offset)

    def get_topic(self, topicName):
        """get topic 
            @type topicName :string 
            @param topicName 
            @return Topic object
        """
        return Topic(topicName, self.cmq_client, self.debug)
    
    def list_topic(self, searchWord="", limit=-1, offset=""):
        
        """ 列出Account的主题

            @type searchWord: string
            @param searchWord: 主题名的前缀

            @type limit: int
            @param limit: list_topic最多返回的主题数

            @type offset: string
            @param offset: list_topic的起始位置，上次list_topic返回的next_offset

            @rtype: tuple
            @return: topicURL的列表和下次list topic的起始位置; 如果所有topic都list出来，next_offset为"".
        """
        params = {}
        if searchWord != "":
            params['searchWord'] = searchWord
        if limit != -1:
            params['limit'] = limit
        if offset != "":
            params['offset'] = offset
        ret_pkg = self.cmq_client.list_topic(params)

        if offset == "":
            next_offset = len(ret_pkg['topicList'])
        else:
            next_offset = offset + len(ret_pkg['topicList'])
        if next_offset >= ret_pkg['totalCount']:
            next_offset = ""

        return (ret_pkg['totalCount'], ret_pkg['topicList'], next_offset)

    
    def get_subscription(self, topicName, subscriptionName):
        ''' 获取订阅
            @type topicName :string
            @param topicName
            
            @type subscriptionName :string
            @param subscriptionName:
            
            @return Subscription object
        
        '''
        return Subscription(topicName, subscriptionName, self.cmq_client, self.debug)
    
    
    

    
    def debuginfo(self, RequestId):
        if self.debug:
            print("===================DEBUG INFO===================")
            print("RequestId: %s" % RequestId)
            print("================================================")
      
	



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

setup(
    name='abmn_qcloud-cmq-sdk-py3',
    version=0.1,
    description='Tencent Cloud SDK for CMQ API 2.0 running in python3',
    author='Tencent Cloud CMQ Team',
    url='https://github.com/tencentyun/cmq-python-sdk/tree/python3',
    maintainer_email="yorkxyzhang@tencent.com",
    scripts=[],
    packages=find_packages(),
    license="Apache License 2.0",
    platforms='any',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ],
)



#!/usr/bin/python
# -*- coding: utf-8 -*-


import time
import base64
from mailcap import subst

class Subscription:
    def __init__(self, topicName, subscriptionName, cmq_client, debug=False):
        self.topicName = topicName
        self.subscriptionName = subscriptionName 
        self.cmq_client = cmq_client
        self.set_encoding(False)
        self.debug = debug

    def set_debug(self, debug):
        self.debug = debug

    def set_encoding(self, encoding):
        """ 设置是否对消息体进行base64编码
 
            @type encoding: bool
            @param encoding: 是否对消息体进行base64编码
        """
        self.encoding = encoding
        
    def create(self, subscription_meta):
        """ 创建订阅

            @type queue_meta: QueueMeta object
            @param queue_meta: QueueMeta对象，设置队列的属性
        """
        params = {
            'topicName' : self.topicName,
            'subscriptionName':self.subscriptionName,
            'endpoint':subscription_meta.Endpoint,
            'protocol' :subscription_meta.Protocal,
            
            'notifyStrategy':subscription_meta.NotifyStrategy,
            'notifyContentFormat':subscription_meta.NotifyContentFormat,
        }
        if subscription_meta.bindingKey :
            n = 1 
            for tag in subscription_meta.bindingKey:
                key = 'bindingKey.%d' % n 
                params[key] = tag
                n += 1
         
        if subscription_meta.FilterTag:
            n = 1 
            for tag in subscription_meta.FilterTag:
                key = 'filterTag.%d' % n
                params[key] = tag
                n += 1         
        self.cmq_client.create_subscription(params)

    def get_attributes(self):
        """ 获取订阅属性

            @rtype: Subscription object
            @return 订阅属性
        """
        params = {
            'topicName':self.topicName,
            'subscriptionName':self.subscriptionName
        }
        resp = self.cmq_client.get_subscription_attributes(params)
        subscripution_meta = SubscriptionMeta()
        self.__resp2meta__(subscripution_meta, resp)
        return subscripution_meta

    def set_attributes(self, subscription_meta):
        """ 设置队列属性

            @type subscription_meta: 
            @param subscription_meta: 订阅属性对象，设置订阅属性
        """
        params = {
            'topicName':self.topicName,
            'subscriptionName':self.subscriptionName,
            }
        if subscription_meta.NotifyStrategy != "":
            params['notifyStrategy'] = subscription_meta.NotifyStrategy
        if subscription_meta.NotifyContentFormat != "":
            params['notifyContentFormat'] = subscription_meta.NotifyContentFormat

        if subscription_meta.bindingKey :
            n = 1 
            for tag in subscription_meta.bindingKey:
                key = 'bindingKey.%d' % n 
                params[key] = tag
                n += 1
         
        if subscription_meta.FilterTag :
            n = 1 
            for tag in subscription_meta.FilterTag:
                key = 'filterTag.%d' % n 
                params[key] = tag
                n += 1
                
        self.cmq_client.set_subscription_attributes(params)
            
    def delete(self):
        """ 删除订阅

        """
        params = {
            'topicName':self.topicName,
            'subscriptionName':self.subscriptionName}        
        self.cmq_client.delete_subscription(params)

    def clearFilterTags(self):
        """ clear filter tags

        """
        params = {
            'topicName':self.topicName,
            'subscriptionName':self.subscriptionName}
        
        self.cmq_client.clear_filterTags(params)
        
        
          
    def __resp2meta__(self, subscription_meta, resp):
        if 'topicOwner' in list(resp.keys()):
            subscription_meta.TopicOwner = resp['topicOwner']
        if 'endpoint' in list(resp.keys()):
            subscription_meta.maxMsgHeapNum = resp['endpoint']
        if 'protocal' in list(resp.keys()):
            subscription_meta.Protocal = resp['protocal']
        if 'notifyStrategy' in list(resp.keys()):
            subscription_meta.NotifyStrategy = resp['notifyStrategy']
        if 'notifyContentFormat' in list(resp.keys()):
            subscription_meta.NotifyContentFormat = resp['notifyContentFormat']
        if 'createTime' in list(resp.keys()):
            subscription_meta.createTime = resp['createTime']
        if 'lastModifyTime' in list(resp.keys()):
            subscription_meta.lastModifyTime = resp['lastModifyTime']
        if 'msgCount' in list(resp.keys()):
            subscription_meta.inactiveMsgNum = resp['msgCount']      
        if 'filterTags' in list(resp.keys()) and resp['filterTags'] is not None:
            for tag in resp['filterTags']:
                subscription_meta.FilterTag.append(tag)
        if 'bindingKey' in list(resp.keys()) and resp['bindingKey'] is not  None:
            for tag in resp['bindingKey']:
                subscription_meta.bindingKey.append(tag)
                

class SubscriptionMeta:
    def __init__(self):
        """ 订阅属性
            :: TopicOwner: 订阅的主题所有者appid
            :: Endpoint: 订阅的终端地址
                                             接收通知的endpoint，根据协议Protocol区分：
                                             对于http，endpoint必须以“http://”开头；
                                             对于https，endpoint必须以“https://”开头；
                                             对于queue，则填queueName。
            :: Protocal: 订阅的协议
            :: NotifyStrategy: 向Endpoint 推送消息的重试策略
                                             描述了向 Endpoint 推送消息出现错误时的重试策略。
                  BACKOFF_RETRY 或者 EXPONENTIAL_DECAY_RETRY，
                                            默认为BACKOFF_RETRY，重试策略的具体描述请参考 基本概念/NotifyStrategy
            :: NotifyContentFormat:  向endpoint端推送的消息格式
            :: FilterTage :该订阅中消息过滤的标签内容
            :: CreateTime :创建时间
            :: LastModifyTime :上次修改时间
            :: msgCount 消息数量
            

        """
        
        self.TopicOwner = ""
        self.Endpoint = ""
        self.Protocal = ""
        self.NotifyStrategy = "EXPONENTIAL_DECAY_RETRY"
        self.NotifyContentFormat = "JSON"
        self.FilterTag = []
        self.CreateTime = 0
        self.LastModifyTime = 0
        self.msgCount = 0     
        self.bindingKey=[]
    def __str__(self):
        meta_info = {"topicOwner" : self.TopicOwner,
                     "endpoint" : self.Endpoint,
                     "protocal" : self.Protocal,
                     "notifyStrategy" : self.NotifyStrategy,
                     "notifyContentFormat" : self.NotifyContentFormat,
                     "filterTag" : self.FilterTag,
                     "createTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.createTime)),
                     "lastModifyTime" : time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(self.lastModifyTime)),
                     "msgCount":self.msgCount,
                     "bindingKey": self.bindingKey,}
        return "\n".join(["%s: %s" % (k.ljust(30), v) for k, v in list(meta_info.items())])


    
    



#!/usr/bin/python
# -*- coding: utf-8 -*-

class CMQExceptionBase(Exception):
    """
    @type code: int
    @param code: 错误类型

    @type message: string
    @param message: 错误描述
    
    @type data: dict
    @param data: 错误数据
    """
    def __init__(self, message, code=-1, data={}):
        self.code = code
        self.message = message
        self.data = data

    def get_info(self):
        return 'Code:%s, Message:%s, Data:%s\n' % (self.code, self.message, self.data)

    def __str__(self):
        return "CMQExceptionBase  %s" % (self.get_info())

class CMQClientException(CMQExceptionBase):
    def __init__(self, message, code=-1, data={}):
        CMQExceptionBase.__init__(self, message, code, data)

    def __str__(self):
        return "CMQClientException  %s" % (self.get_info())
    
class CMQClientNetworkException(CMQClientException):
    """ 网络异常

        @note: 检查endpoint是否正确、本机网络是否正常等;
    """
    def __init__(self, message, code=-1, data={}):
        CMQClientException.__init__(self, message, code, data)

    def __str__(self):
        return "CMQClientNetworkException  %s" % (self.get_info())

class CMQClientParameterException(CMQClientException):
    """ 参数格式错误

        @note: 请根据提示修改对应参数;
    """
    def __init__(self, message, code=-1, data={}):
        CMQClientException.__init__(self, message, code, data)

    def __str__(self):
        return "CMQClientParameterException  %s" % (self.get_info())

class CMQServerNetworkException(CMQExceptionBase):
    """ 服务器网络异常
    """
    def __init__(self, status = 200, header = None, data = ""):
        if header == None:
            header = {}
        self.status = status
        self.header = header
        self.data = data

    def __str__(self):
        return "CMQServerNetworkException Status: %s\nHeader: %s\nData: %s\n" % \
            (self.status, "\n".join(["%s: %s" % (k,v) for k,v in list(self.header.items())]), self.data)

class CMQServerException(CMQExceptionBase):
    """ cmq处理异常

        @note: 根据code进行分类处理，常见错误类型：
             : 4000       参数不合法
             : 4100       鉴权失败:密钥不存在/失效
             : 4300       账户欠费了
             : 4400       消息大小超过队列属性设置的最大值
             : 4410       已达到队列最大的消息堆积数
             : 4420       qps限流
             : 4430       删除消息的句柄不合法或者过期了
             : 4440       队列不存在
             : 4450       队列个数超过限制
             : 4460       队列已经存在
             : 6000       服务器内部错误
             : 6010       批量删除消息失败（具体原因还要看每个消息删除失败的错误码）
             : 7000       空消息，即队列当前没有可用消息
             : 更多错误类型请登录腾讯云消息服务官网进行了解；
    """
    def __init__(self, message, request_id, code=-1, data={}):
        CMQExceptionBase.__init__(self, message, code, data)
        self.request_id = request_id

    def __str__(self):
        return "CMQServerException  %s\nRequestID:%s" % (self.get_info(), self.request_id)





